<?php

namespace Drupal\identification\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Path\PathValidatorInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form to configure identfication settings for this site.
 */
class IdentificationConfigForm extends ConfigFormBase {


  /**
   * The path validator.
   *
   * @var \Drupal\Core\Path\PathValidatorInterface
   */
  protected $pathValidator;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new IdentificationConfigForm.
   *
   * @param \Drupal\Core\Path\PathValidatorInterface $path_validator
   *   The path validator service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   */
  public function __construct(PathValidatorInterface $path_validator, EntityFieldManagerInterface $entity_field_manager, ConfigFactoryInterface $config_factory) {
    $this->pathValidator = $path_validator;
    $this->entityFieldManager = $entity_field_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
    $container->get('path.validator'),
    $container->get('entity_field.manager'),
    $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'identification_configuration_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['identification.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('identification.settings');
    $url_object = $this->pathValidator->getUrlIfValid('admin/config/people/accounts/fields');
    $route_name = $url_object->getRouteName();
    // Create URL.
    $url = Url::fromRoute($route_name);
    $form['identification_field_name'] = [
      '#type' => 'select',
      '#title' => $this->t('Identification field'),
      '#field_suffix' => Link::fromTextAndUrl($this->t('Create new field'), $url),
      '#default_value' => $config->get('identification_field_name'),
      '#description' => $this->t('Select the field to use for the user identification number (or string).'),
      '#required' => FALSE,
    ];

    // We'll allow the following field types to be used as an identifier.
    $field_types = [
      'decimal',
      'float',
      'integer',
      'string',
    ];

    // Grab list of eligible field names and place them as options on the form.
    $options = ['' => '- None -'];
    foreach ($this->entityFieldManager->getFieldDefinitions('user', 'user') as $field_name => $field_definition) {
      $field_definition_bundle = $field_definition_type = NULL;
      $field_definition_bundle = $field_definition->getTargetBundle();
      $field_definition_type = $field_definition->getType();
      if (!empty($field_definition_bundle) && in_array($field_definition_type, $field_types)) {
        $options[$field_name] = $field_definition->getLabel() . " ({$field_name})";
      }
    }
    $form['identification_field_name']['#options'] = $options;
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable('identification.settings');
    $config->set('identification_field_name', $form_state->getValue('identification_field_name'));
    $config->save();
    parent::submitForm($form, $form_state);
  }

}
